#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#define IPVERSION(x) x->ai_family==AF_INET?"IPv4":x->ai_family==AF_INET6?"IPv6":"?"

int main(const int ParmCount, const char* Parameter[]) {
  char Port[6];
  int StartPort=0;
  int EndPort=0;
  int sock;
  uint16_t s;
  int n;
  struct addrinfo hints;
  struct addrinfo* res;
  struct addrinfo* r;

  if((ParmCount < 3) || (ParmCount < 3)) {
    fprintf(stderr,"usage: openports hostname [start]port [endport]\n");
    return 1;
  }
  else if(ParmCount == 3) {
    StartPort = atoi(Parameter[2]);
    if((StartPort < 0) || (StartPort > 65535)) {
      fprintf(stderr,"Port out of range 0..65535\n");
      return 1;
    }
    EndPort = StartPort;
  }
  else if(ParmCount == 4) {
    StartPort = atoi(Parameter[2]);
    EndPort = atoi(Parameter[3]);
    if((StartPort < 0) || (StartPort > 65535)) {
      fprintf(stderr,"Start port out of range 0..65535\n");
      return 1;
    }
    else if((EndPort < 0) || (EndPort > 65535)) {
      fprintf(stderr,"End port out of range 0..65535\n");
      return 1;
    }
    else if(EndPort < StartPort) {
      fprintf(stderr,"End port must be above start port\n");
      return 1;
    }
  }

  memset(&hints, 0, sizeof(hints));
  hints.ai_family = AF_UNSPEC;
  hints.ai_socktype = SOCK_STREAM;

  for(s=StartPort;s<=EndPort;s++) {
    sprintf(Port, "%d", s);

    if((n = getaddrinfo(Parameter[1], Port, &hints, &res)) != 0) {
      fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(n));
    return 1;
    }

    for(r=res; r != NULL; r = r->ai_next) {
      if((sock = socket(r->ai_family, r->ai_socktype, r->ai_protocol)) < 0) {
        continue;
      }
      if (connect(sock, r->ai_addr, r->ai_addrlen) < 0) {
        if(errno == ECONNREFUSED)
          printf("%d/%s: closed\n", s, IPVERSION(r));
        else if(errno != EHOSTUNREACH)
          printf("%d/%s: %s\n", s, IPVERSION(r), strerror(errno));
        close(sock);
        continue;
      }
      printf("%d/%s: open\n", s, IPVERSION(r));
      close(sock);
    }
    freeaddrinfo(res);
  }

  return 0;
}

