#include <linux/version.h>
#include <errno.h>
#include <time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#define PORT "13"
#define MAXPENDING 2

#ifndef SO_REUSEPORT
#define SO_REUSEPORT 15
#endif

int main(const int ParmCount, const char* Parameter[]) {
  const char* Host = NULL;
  int sock;
  int csock;
  struct sockaddr client_addr;
  socklen_t client_addrlen;
  int n;
  int yes = 1;
  time_t now;
  struct addrinfo hints;
  struct addrinfo* res;
  struct addrinfo* r;

  if(ParmCount > 2) {
    fprintf(stderr,"usage: sendtime [hostname|ip]\n");
    return 1;
  }
  else if(ParmCount == 2) {
    Host = Parameter[1];
  }

  memset(&hints, 0, sizeof(hints));
  hints.ai_family = AF_UNSPEC;
  hints.ai_socktype = SOCK_STREAM;
  hints.ai_flags = AI_PASSIVE;

  if((n = getaddrinfo(Host, PORT, &hints, &res)) != 0) {
    fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(n));
    return 1;
  }

  for(r=res; r != NULL; r = r->ai_next) {
    if((sock = socket(r->ai_family, r->ai_socktype, r->ai_protocol)) < 0) {
      fprintf(stderr, "can't create socket: %s\n", strerror(errno));
      continue;
    }
    if (setsockopt(sock, SOL_SOCKET, SO_REUSEPORT, &yes, sizeof(int)) < 0) {
      if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) < 0) {
        fprintf(stderr, "can't set socket options: %s\n", strerror(errno));
        continue;
      }
    }
    if (bind(sock, r->ai_addr, r->ai_addrlen) < 0) {
      close(sock);
      fprintf(stderr, "can't bind to socket: %s\n", strerror(errno));
      continue;
    }
    if (listen(sock, MAXPENDING) < 0) {
      fprintf(stderr, "can't listen to socket: %s\n", strerror(errno));
      continue;
    }

    if(!fork()) {
      while(1) {
        client_addrlen = sizeof(client_addr);
        csock = accept(sock, &client_addr, &client_addrlen);
        now = time(NULL);
        if (send(csock, ctime(&now), 24, 0) < 0) {
          fprintf(stderr, "can't send data: %s\n", strerror(errno));
          close(csock);
          continue;
        }
        close(csock);
      }
    }
    close(sock);
  }
  freeaddrinfo(res);

  return 0;
}

